\name{regionswrapper}
\alias{regionswrapper}
\title{
Differential methylation testing on all 11 categories of annotated region
}
\description{
This function will output all the differential methylation testing results for the 11
categories of annotated regions, and store the results in the separated sheets of an excel
file.
}
\usage{
regionswrapper(dataf, indexmethod = c("mean", "median", "tbrm"), gcase = "g2", 
gcontrol = "g1", testmethod = c("wilcox", "limma", "pooled", "satterthwaite"), 
Padj = "BH", concov = c("OFF", "ON"), paired = FALSE, list11excel, list11Rdata, 
rawpcut = NULL, adjustpcut = NULL, betadiffcut = NULL)
}
\arguments{
   \item{dataf}{
a \emph{methy450batch} class returned by the \emph{IMA.methy450PP} function
}
  \item{indexmethod}{
the methods available to derive an index of overall methylation level for each region
}
  \item{gcase}{
the string names of case group/groups in the sample.txt file
}
  \item{gcontrol}{
the string names of control group/groups in the sample.txt file
}
  \item{testmethod}{
the available methods include wilcoxon rank-sum test (default), Student's t-test ("pooled" or "satterthwaite") or empirical Bayes statistics.
}
  \item{Padj}{
the methods available for multiple testing correction. Users could choose any method provided in the \emph{p.adjust} function of R \emph{stat} package.
}
  \item{concov}{
if "ON", covariates is continuous variable, and the linear model would be used for testing the association between methylation level and continuous phenotype (e.g., age).
}
\item{paired}{
if TRUE, the test method would change to the corresponding paired-test method
%%     ~~Describe \code{paired} here~~
}
  \item{list11excel}{
the name of output excel file containing the differential testing results for each annotated region
}
  \item{list11Rdata}{
the name of output Rdata containing the differential testing result for each annotated region
}
  \item{rawpcut}{
either null or the output cut-off for the raw p-value
}
\item{adjustpcut}{
either null or the output cut-off for the adjusted p-value
}
  \item{betadiffcut}{
either null or the output cut-off for the \eqn{beta} value difference between two groups
}

}
\value{
Return an excel file containing the differential testing result for each annotated region as well as an Rdata file containing the differential testing result for each annotated region
}
\author{
Dan Wang, Li Yan, Qiang Hu, Dominic J Smiraglia, Song Liu

Maintainer: Dan Wang <wangdan412@gmail.com>
}

\seealso{
  \code{\link{IMA.methy450PP}},\code{\link{IMA.methy450R}},\code{\link{testfunc}},\code{\link{sitetest}},\code{\link{indexregionfunc}},\code{\link{annotfunc}}

}
\examples{
setwd(system.file("extdata", package="IMA"))
MethyFileName = "SampleMethFinalReport.txt"
PhenoFileName = "SamplePhenotype.txt"
data = IMA.methy450R(fileName = MethyFileName,columnGrepPattern=list(beta=".AVG_Beta",detectp=".Detection.Pval"),groupfile = PhenoFileName);
dataf = IMA.methy450PP(data,na.omit = TRUE,normalization=FALSE,transfm = FALSE,samplefilterdetectP = 1e-5,samplefilterperc = 0.75,sitefilterdetectP = 0.05,sitefilterperc=0.75,locidiff = FALSE, XYchrom = FALSE,snpfilter=FALSE ) 
regionswrapper(dataf,indexmethod="mean",gcase="g2",gcontrol="g1",testmethod="limma",Padj="BH",concov="OFF",paired=FALSE,list11excel="./test1.xls",list11Rdata="./test.Rdata",rawpcut = NULL,adjustpcut =NULL,betadiffcut = NULL)
}
